/*
 * ==========================================================================================
 * benchmark-mysql.c
 *
 * Date: 09 July 2003
 * Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
 * Version: 2.0 
 * ------------------------------------------------------------------------------------------
 *
 * Interface between mod_benchmark and mysql.
 * To implement other DB, just create another .so with
 * same functions.
 *
 * ------------------------------------------------------------------------------------------
 * CHANGES:
 *
 * ------------------------------------------------------------------------------------------
 * TODO:
 * 
 * ==========================================================================================
 */
static const char *rcsid = "$Id$";

#include "common.h"
#include <httpd.h>
#include <mysql.h>
#include <stdio.h>

typedef struct {
	MYSQL *init;
	MYSQL *conn;
} opaque_t;

/*
 * Init function: 
 * Allocate some private data and connect to the DB
 */
const void *db_init(apr_pool_t *pool, const char *srv, const char *db, const char *user, const char *pwd)
{
	MYSQL *init;
	MYSQL *conn;
 
	if (!(init = mysql_init(NULL))) {
		fprintf(stderr,"mysql_init() failed\n");
		return NULL;
	}

	conn = mysql_real_connect(	init,
					(*srv)?srv:NULL,
					(*user)?user:NULL,
					(*pwd)?pwd:NULL,
					db,
					0,	/* default port (UNIX socket !) */
					NULL,	/* default scoket name */
					0);	/* no flags */

	if (!conn) {
		fprintf(stderr,"mysql_real_connect() failed: %s\n",mysql_error(init));
		mysql_close(init);
		return NULL;
	}

	return init;
}

/*
 * Called each time a row must be inserted into the Log table:
 */
int db_insert(const void *opaque, const char *query)
{
	int ret = mysql_query((MYSQL *)opaque,query);

	if (ret) 
		fprintf(stderr,"mysql_query(%s) failed: %s\n",query,mysql_error((MYSQL *)opaque));

	return ret;
}

/*
 * Called when apache exits:
 */
void db_close(const void *opaque)
{
	mysql_close((MYSQL *)opaque);
}

/*
 * $Log:$
 */

/* EOF */
