/*
 * ==========================================================================================
 * benchmark-pgsql.c
 *
 * Date: 09 July 2003
 * Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
 * Version: 2.0 
 * ------------------------------------------------------------------------------------------
 *
 * Interface between mod_benchmark and PostgreSQL.
 * To implement other DB, just create another .so with
 * same functions.
 *
 * ------------------------------------------------------------------------------------------
 * CHANGES:
 *
 * ------------------------------------------------------------------------------------------
 * TODO:
 * 
 * ==========================================================================================
 */
static const char *rcsid = "$Id$";

#include "common.h"
#include <httpd.h>
#include <libpq-fe.h>
#include <stdio.h>

/*
 * Init function: 
 * Allocate some private data and connect to the DB
 */
const void *db_init(apr_pool_t *pool, const char *srv, const char *db, const char *user, const char *pwd)
{
	PGconn *conn;
	char   *info = "";

	if (srv)  	   info = apr_pstrcat(pool,"host='",srv,"' ",NULL);
	if (db)   	   info = apr_pstrcat(pool,info,"dbname='",db,"' ",NULL);
	if (user && *user) info = apr_pstrcat(pool,info,"user='",user,"' ",NULL);
	if (pwd && *pwd)   info = apr_pstrcat(pool,info,"password='",pwd,"'",NULL);
 
	if (!(conn = PQconnectdb(info))) {
		fprintf(stderr,"PGconnectdb() failed\n");
		return NULL;
	}

	if (PQstatus(conn) != CONNECTION_OK) {
		fprintf(stderr,"Connection failed: %s\n",PQerrorMessage(conn));
		return NULL;
	}

	return conn;
}

/*
 * Called each time a row must be inserted into the Log table:
 */
int db_insert(const void *opaque, const char *query)
{
	PGresult *res = PQexec((PGconn *)opaque,query);

	if (!res) {
		fprintf(stderr,"PQexec(%s) failed: %s\n",query,PQerrorMessage((PGconn *)opaque));
	}
	else {
		ExecStatusType status = PQresultStatus(res);
		if (status != PGRES_COMMAND_OK) {
			fprintf(stderr,"PQexec(%s) failed: %s\n",query,PQresStatus(status));
		}

		PQclear(res);
	}

	return 0;
}

/*
 * Called when apache exits:
 */
void db_close(const void *opaque)
{
	PQfinish((PGconn *)opaque);
}

/*
 * $Log:$
 */

/* EOF */
