/*
 * ==========================================================================================
 * hash.c
 *
 * Date: 09 July 2003
 * Author: Jerome Delamarche (c) Copyright INODES.  2003  All rights reserved.
 * Version: 2.0 
 * ------------------------------------------------------------------------------------------
 *
 * Hash array implementation
 * This is a specific implementation for benchmark_rt.c, not a generic one !
 *
 * ------------------------------------------------------------------------------------------
 * CHANGES:
 *
 * ------------------------------------------------------------------------------------------
 * TODO:
 *
 * ==========================================================================================
 */
static const char *rcsid = "$Id$";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "hash.h"

/*
 * --------------------------------------------
 * Create a new hash array:
 * --------------------------------------------
 */
MYHASH *HashAllocate()
{
	MYHASH *h = (MYHASH *)malloc(sizeof(MYHASH));

	memset(h,0,sizeof(MYHASH));

	return h;
}

/*
 * --------------------------------------------
 * Release the hash structure:
 * --------------------------------------------
 */
void HashFree(MYHASH *hash)
{
	int i;

	for ( i = 0 ; i < HASHSIZE ; i++ ) {
		MYHASHENTRY *entry = (*hash)[i];
		while (entry) {
			MYHASHENTRY *next = entry->next;
			free((void *)entry->key);
			free((void *)entry->data);
			free(entry);
			entry = next;
		}
	}

	free(*hash);
}

/*
 * --------------------------------------------
 * Get the entry associated with a hash key:
 * --------------------------------------------
 */
void *HashGetEntryOf(MYHASH *hash, const char *key)
{
	int hashvalue = HashValue(key);
	MYHASHENTRY *entry = (*hash)[hashvalue];

	/* loop on the hash table entry to take care of collisions: */
	while (entry) {
		if (!strcmp(key,entry->key))
			return entry->data;

		entry = entry->next;
	}

	return NULL;
}

/*
 * --------------------------------------------
 * Enter a new key in the hash table:
 * --------------------------------------------
 */
void *HashAddEntry(MYHASH *hash, const char *key, int size)
{
	/* it is the caller responsability to ensure the
	 * key has not been already inserted:
	 */
	int hashvalue = HashValue(key);
	MYHASHENTRY *newentry = (MYHASHENTRY *)malloc(sizeof(MYHASHENTRY));

	newentry->data = malloc(size);
	newentry->next = (*hash)[hashvalue];
	newentry->key = strdup(key);

	(*hash)[hashvalue] = newentry;

	return newentry->data;
}

/*
 * -----------------------------------------------------------------
 * Hash coding function (must return a value between 0 and HASHSIZE):
 * -----------------------------------------------------------------
 */
int HashValue(const char *key)
{
	/* takes the modulo HASHSIZE of the sum of the key chars:
	 * (could we find something better ?)
	 */
	char *p = (char *)key;
	int val = 0;

	while (*p) {
		val += *p++;
	}

	return val % HASHSIZE;
}

/*
 * -------------------------------------------
 * Dump the specified hash table:
 * -------------------------------------------
 */
void HashDump(MYHASH *hash, void(*cback)(int, const char *, void *, void *), void *opaque)
{
	int i;

	for ( i = 0 ; i < HASHSIZE ; i++ ) {
		MYHASHENTRY *entry = (*hash)[i];
		if (!entry) continue;

		while (entry) {
			(*cback)(i,entry->key,entry->data,opaque);
			entry = entry->next;
		}
	}
}

/* EOF */
